SystemOrganization addCategory: #'CodeMine-Grammar'!
SystemOrganization addCategory: #'CodeMine-Text'!
SystemOrganization addCategory: #'CodeMine-LanguageModel'!
SystemOrganization addCategory: #CodeMine!

Object subclass: #CMCallGraph
	instanceVariableNames: 'selectorsMap methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallGraph class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 20:32'!
fromCategory: aString
	"Compute call graph from categories beginning with aString"

	| callGraph |
	
	callGraph := CMCallGraph new.
	(CMCallGraphBuilder forGraph: callGraph) scanCategoryPrefix: aString.
	^callGraph! !

!CMCallGraph class methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:09'!
fromCategory: aString excluding: aBlock
	"Compute call graph from categories beginning with aString"

	| callGraph |
	
	callGraph := CMCallGraph new.
	(CMCallGraphBuilder forGraph: callGraph) scanCategoryPrefix: aString excluding: aBlock.
	^callGraph! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:54'!
addMethod: aCMMethod
	methods add: aCMMethod
	
	! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:07'!
initialize 
	super initialize.
	selectorsMap := Dictionary new.
	methods := OrderedCollection new.! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
methods
	^ methods! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
methods: anObject
	methods := anObject! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:14'!
selectorFor: aSymbol
	
	^selectorsMap
		at: aSymbol 
		ifAbsentPut: [ CMSelector named: aSymbol ]! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selectorsMap
	^ selectorsMap! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selectorsMap: anObject
	selectorsMap := anObject! !

Object subclass: #CMCallingRelation
	instanceVariableNames: 'previous next container selector backwardDeps forwardDeps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
backwardDeps
	^ backwardDeps! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
backwardDeps: anObject
	backwardDeps := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
container
	^ container! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
container: anObject
	container := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
forwardDeps
	^ forwardDeps! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:31'!
forwardDeps: anObject
	forwardDeps := anObject! !

!CMCallingRelation methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 14:32'!
initialize

	backwardDeps := OrderedCollection new.
	forwardDeps := OrderedCollection new.! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selector
	^ selector! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selector: anObject
	selector := anObject! !

Object subclass: #CMCodeTopicModel
	instanceVariableNames: 'methodTexts featureSpace topicModel linkField'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Text'!

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 18:42'!
computeLinkedTopics: numTopics forIterations: iterations
	
	| lda |

	lda := LDLinkedAllocator new.
	lda linkField: linkField;
		contextWeight: 0.3;
		featurePriors: (self featureSpace estimateFeaturePriors).
	
	self topicModel: (lda
		allocate: (self methodTexts values) 
		iterating: iterations 
		features: (self featureSpace featureCount) 
		topics: numTopics).
	^self topicModel
	
	! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:41'!
computeTopics: numTopics forIterations: iterations
	
	| lda |

	lda := LDAllocator new.
	self topicModel: (lda
		allocate: (self methodTexts values) 
		iterating: iterations 
		features: (self featureSpace featureCount) 
		topics: numTopics).
	^self topicModel
	
	! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 17:38'!
extractTextFromCallGraph: aCallGraph
	"Collect all textual tokens from a call graph, eliminate non-frequent tokens
	and construct the feature space from the rest"

	| frequencies methodFeatures dependencies selectorMapping docId |
	frequencies := Dictionary new.
	methodFeatures := OrderedDictionary new.
	dependencies := Dictionary new.
	selectorMapping := Dictionary new.

	docId := 1.
	
	aCallGraph methods do: [ :method | | textVisitor currentSelector |
		currentSelector := method implementedSelector symbol.
		selectorMapping at: currentSelector ifAbsentPut: [OrderedCollection new].
		(selectorMapping at: currentSelector) add: docId.
			
		textVisitor := CMTextVisitor new.
		textVisitor onSelector: [ :position :targetSelector | 
			dependencies at: (docId @ position) put: targetSelector.
		].

		textVisitor visitNode: (method sourceTree).
		textVisitor literals do: [ :symbol |
			frequencies at: symbol put: (1 + (frequencies at: symbol ifAbsentPut: 0)).
		].
		methodFeatures at: method put: (textVisitor literals).
		docId := docId + 1.
	].

	methodFeatures keysAndValuesDo: [ :method :features | | reducedFeatures |
		reducedFeatures := features select: [:feature | (frequencies at: feature) > 2 ].
		self methodTexts
			at: method 
			put: (self featureSpace representAll: reducedFeatures).
		].

	"Unwrap 
		(d @ w) -> #selector
	and
		#selector -> d1, dw, ...
	to
		(d @ w) -> d1, d2, ..."

	linkField := LDLinkField new.
	dependencies keysAndValuesDo: [ :position :targetSelector |
		selectorMapping at: targetSelector ifPresent: [ :targetIds |
			targetIds do: [ :id |  linkField linkDocument: (position x) at: (position y) to: id ]]].! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
featureSpace
	^ featureSpace! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
featureSpace: anObject
	featureSpace := anObject! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 18:42'!
initialize 
	
	methodTexts := OrderedDictionary new.
	featureSpace := LDDocumentFeatureSpace new.! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:30'!
methodTexts
	^ methodTexts! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:30'!
methodTexts: anObject
	methodTexts := anObject! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:53'!
sortedTopic: topic
	
	^(((self topicModel featuresForTopicAt: topic) 
		withIndexCollect: [ :value :index | 
			Array with: value with: index]) 
		sorted: [ :a :b | (a first) > (b first) ])
		collect: [ :pair | self featureSpace interpret: (pair second) ]
	! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
topicModel
	^ topicModel! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
topicModel: anObject
	topicModel := anObject! !

Object subclass: #CMConceptPart
	instanceVariableNames: 'extent intent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

Object subclass: #CMDataDependency
	instanceVariableNames: 'name definition uses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

CMDataDependency subclass: #CMArgumentDependency
	instanceVariableNames: 'sources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMArgumentDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:48'!
named: aString
	
	^(CMArgumentDependency new)
		name: aString;
		yourself.! !

!CMArgumentDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isArgument

	^ true! !

!CMArgumentDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:41'!
sources
	^ sources! !

!CMArgumentDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:41'!
sources: anObject
	sources := anObject! !

!CMDataDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 14:42'!
definedAt: aCallingRelation
	
	^(CMDataDependency new)
		definition: aCallingRelation;
		uses: OrderedCollection new;
		yourself.! !

!CMDataDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 14:43'!
named: aString definedAt: aCallingRelation
	
	^(CMDataDependency new)
		definition: aCallingRelation;
		uses: OrderedCollection new;
		name: aString;
		yourself.! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
definition
	^ definition! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
definition: anObject
	definition := anObject! !

!CMDataDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isArgument

	^ false! !

!CMDataDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isSelf

	^ false! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:43'!
name
	^ name! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:43'!
name: anObject
	name := anObject! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
uses
	^ uses! !

!CMDataDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 14:41'!
uses: anObject
	uses := anObject! !

CMDataDependency subclass: #CMSelfDependency
	instanceVariableNames: 'cls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMSelfDependency class methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 17:22'!
newForClass: aClass

	^(CMSelfDependency new)
		cls: aClass;
		name: 'self';
		yourself! !

!CMSelfDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:34'!
cls
	^ cls! !

!CMSelfDependency methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:35'!
cls: anObject
	cls := anObject! !

!CMSelfDependency methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:49'!
isSelf

	^ true! !

Object subclass: #CMLNode
	instanceVariableNames: 'topic parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLNode subclass: #CMLAbstractNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLAbstractNode subclass: #CMLSomeMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLAbstractNode subclass: #CMLSomeValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLNode subclass: #CMLConcreteNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLConcreteNode subclass: #CMLMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLConcreteNode subclass: #CMLReturn
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLConcreteNode subclass: #CMLSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLConcreteNode subclass: #CMLValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLValue subclass: #CMLArray
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

!CMLArray methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:56'!
values

	^ values! !

!CMLArray methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:56'!
values: anObject

	values := anObject! !

CMLValue subclass: #CMLAssignment
	instanceVariableNames: 'variable value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLValue subclass: #CMLBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLValue subclass: #CMLLiteral
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLLiteral subclass: #CMLLiteralArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLLiteral subclass: #CMLNumber
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLLiteral subclass: #CMLString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLLiteral subclass: #CMLSymbol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLValue subclass: #CMLMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

CMLValue subclass: #CMLVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

Object subclass: #CMLSimpleModel
	instanceVariableNames: 'instances'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

!CMLSimpleModel methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 16:06'!
addInstance: aCMLNode
! !

!CMLSimpleModel methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:45'!
instances

	^ instances! !

!CMLSimpleModel methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:45'!
instances: anObject

	instances := anObject! !

Object subclass: #CMMethod
	instanceVariableNames: 'implementedSelector callingRelations sourceTree vocabulary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
callingRelations
	^ callingRelations! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
callingRelations: anObject
	callingRelations := anObject! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementedSelector
	^ implementedSelector! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementedSelector: anObject
	implementedSelector := anObject! !

!CMMethod methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 14:53'!
initialize 
	super initialize.
	callingRelations := OrderedCollection new.
	vocabulary := Set new.! !

!CMMethod methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 17:26'!
newCallTo: aSelector 

	| call |
	call := CMCallingRelation new.
	call	selector: aSelector;
		container: self.
	aSelector senders add: call.
	self callingRelations add: call.
	^call
	! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 20:49'!
sourceTree
	^sourceTree! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:08'!
sourceTree: aSourceAst
	sourceTree := aSourceAst! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 1/29/2016 14:47'!
vocabulary

	^ vocabulary! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 1/29/2016 14:47'!
vocabulary: anObject

	vocabulary := anObject! !

Object subclass: #CMSelector
	instanceVariableNames: 'symbol implementors senders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMSelector class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:14'!
named: aSymbol

	^(CMSelector new)
		symbol: aSymbol;
		yourself! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementors
	^ implementors! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementors: anObject
	implementors := anObject! !

!CMSelector methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:13'!
initialize
	implementors := OrderedCollection new.
	senders := OrderedCollection new.! !

!CMSelector methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:08'!
newImplementor: aSourceAst
	
	| method |

	method := CMMethod new.
	method 
		implementedSelector: self;
		sourceTree: aSourceAst.
	self implementors add: method.
	^method
! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
senders
	^ senders! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
senders: anObject
	senders := anObject! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
symbol
	^ symbol! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
symbol: anObject
	symbol := anObject! !

Object subclass: #CXExpression
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

CXExpression subclass: #CXChoice
	instanceVariableNames: 'left right probability'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
left

	^ left! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
left: anObject

	left := anObject! !

!CXChoice methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 18:36'!
parseOn: aParseState

	^(self left parseOn: aParseState) 
		onSuccess: [ :result | 
			(aParseState statsFor: self) goLeft. 
			result ]
		onFailure: [
			(self right parseOn: aParseState) 
			].! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
probability

	^ probability! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
probability: anObject

	probability := anObject! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
right

	^ right! !

!CXChoice methodsFor: 'accessing' stamp: 'toma 9/1/2015 18:28'!
right: anObject

	right := anObject! !

!CXChoice methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:27'!
statClass
	^CXChoiceStat! !

CXExpression subclass: #CXString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

Object subclass: #CXParseResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

CXParseResult subclass: #CXFailure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXFailure methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 18:00'!
onSuccess: successBlock onFailure: failureBlock

	^ failureBlock value! !

!CXParseResult methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 18:29'!
onSuccess: successBlock onFailure: failureBlock
	^ successBlock value: self

	! !

CXParseResult subclass: #CXToken
	instanceVariableNames: 'data from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

Object subclass: #CXParseState
	instanceVariableNames: 'data position stats'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
data

	^ data! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
data: anObject

	data := anObject! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:32'!
initialize

	super initialize.
	self position: 1.
	self stats: (Dictionary new).! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
position

	^ position! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
position: anObject

	position := anObject! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
stats

	^ stats! !

!CXParseState methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:27'!
stats: anObject

	stats := anObject! !

!CXParseState methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:26'!
statsFor: anExpression

	^self stats at: anExpression ifAbsentPut: [anExpression statClass new].! !

Object subclass: #CXStat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

CXStat subclass: #CXChoiceStat
	instanceVariableNames: 'leftCount rightCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Grammar'!

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:22'!
goLeft
	self leftCount: (self leftCount + 1)! !

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:22'!
goRight
	self rightCount: (self rightCount + 1)! !

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:17'!
initialize
	super initialize.
	self
		leftCount: 0;
		rightCount: 0.	! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
leftCount

	^ leftCount! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
leftCount: anObject

	leftCount := anObject! !

!CXChoiceStat methodsFor: 'as yet unclassified' stamp: 'toma 9/1/2015 17:20'!
probability
	| sum |
	"maximum likelihood estimate for P given collected statistics"
	sum := self leftCount + (self rightCount).
	(sum = 0)
		ifTrue: [	 ^ self  leftCount asFloat / sum ]
		ifFalse: [ ^ 0.5 ]
	! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
rightCount

	^ rightCount! !

!CXChoiceStat methodsFor: 'accessing' stamp: 'toma 9/1/2015 17:14'!
rightCount: anObject

	rightCount := anObject! !

RBProgramNodeVisitor subclass: #CMCallGraphBuilder
	instanceVariableNames: 'currentMethod callGraph currentVariable definitions currentSelf controlFlow currentStatement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallGraphBuilder class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 18:02'!
forGraph: aCallGraph
	
	^(CMCallGraphBuilder new)
		callGraph: aCallGraph;
		yourself.
		! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:20'!
acceptMessageNode: aMessageNode
	| selector |


	selector := self callGraph selectorFor: aMessageNode selector.
	self currentStatement: (self currentMethod newCallTo: selector).
	super acceptMessageNode: aMessageNode.

	! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:50'!
acceptMethodNode: aMethodNode
	
	| selector |
	selector := self callGraph selectorFor: (
		aMethodNode selector asSymbol).
	self currentMethod: (selector newImplementor: aMethodNode).
	self definitions: Dictionary new.
	self definitions at: 'self' put: (self currentSelf).
.
	aMethodNode argumentNames do: [ :arg |
		self definitions at: arg put: (CMArgumentDependency named: arg)].

	self callGraph methods add: (self currentMethod).
	super acceptMethodNode: aMethodNode.! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 14:50'!
acceptVariableNode: aVariableNode
	self currentMethod vocabulary add: (aVariableNode name).
	super acceptVariableNode: aVariableNode.! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
callGraph
	^ callGraph! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
callGraph: anObject
	callGraph := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
controlFlow
	^ controlFlow! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
controlFlow: anObject
	controlFlow := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
currentMethod
	^ currentMethod! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
currentMethod: anObject
	currentMethod := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:28'!
currentSelf
	^ currentSelf! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:28'!
currentSelf: anObject
	currentSelf := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:18'!
currentStatement
	^ currentStatement! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:18'!
currentStatement: anObject
	currentStatement := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
definitions
	^ definitions! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/17/2015 15:02'!
definitions: anObject
	definitions := anObject! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:02'!
initialize 

	super initialize.
	definitions := Dictionary new.! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 11:57'!
scanCategoryPrefix: aString

	^self scanCategoryPrefix: aString excluding: [ :class | false ]
! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:33'!
scanCategoryPrefix: aString excluding: aBlock

	((Object allSubclasses
		select: [ :class |
			class category ifNil: [false] ifNotNilDo: [:cat | cat beginsWith: aString]])
		select: [ :class |
			(aBlock value: class) not]) 
		do: [: class |
			self scanClass: class].
! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/17/2015 15:37'!
scanClass: aClass

	self currentSelf: (CMSelfDependency newForClass: aClass).

	aClass methodDict valuesDo: [ :method | 	| node |
		node := RBParser parseMethod: (method getSource asString).
		self visitNode: node.
	]
! !

RBProgramNodeVisitor subclass: #CMCondenser
	instanceVariableNames: 'stream tagStack translation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCondenser class methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
on: aStream

	^(CMTreeExporter new)
		stream: aStream; 
		yourself! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptArrayNode: anArrayNode
	self inside: 'array' do: [
		super acceptArrayNode: anArrayNode]! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptAssignmentNode: aNode
	self inside: 'assignment' do: [
			self inside: 'to' do: [self visitNode:  aNode variable].
			self inside: 'value' do: [self visitNode: aNode value]]
			! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptBlockNode: aNode
	self inside: 'block' do: [
		aNode arguments ifNotEmpty: [
		self inside: 'args' do: [
			self visitArguments: aNode arguments]].
		self inside: 'body' do: [
			self visitNode: aNode body.]]! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptCascadeNode: aNode
	self inside: 'cascade' do: [
		super acceptCascadeNode: aNode]! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptLiteralArrayNode: aNode
	self inside: 'literals' do: [
		super acceptLiteralArrayNode: aNode.]	! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptLiteralNode: aNode
		self text: (aNode value asString) tagged: 'literal'! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptMessageNode: aNode
	self inside: 'message' 
	with: {'selector' -> aNode selector asString}
	do: [	
			(aNode isCascaded not or: [aNode isFirstCascaded]) 
				ifTrue: [
					self inside: 'receiver' do:  [
						self visitNode: aNode receiver]].
			aNode arguments ifNotEmpty: [
				self inside: 'args' do: [
					aNode arguments do: [
						:each | self visitNode: each]]]].! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptMethodNode: aNode
	self inside: 'method' 
	with: {'selector' -> aNode selector asString}
	do: [
		aNode arguments ifNotEmpty: [
			self inside: 'args' do: [
				self visitArguments: aNode arguments]].
		aNode pragmas ifNotEmpty: [
			self inside: 'pragmas' do: [
				aNode pragmas	do: [ :each | self visitNode: each ]]].
		self inside: 'body' do: [
			self visitNode: aNode body]].! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptPatternBlockNode: aNode
	self fail: 'Not Implemented'! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptPatternWrapperBlockNode: aNode
	self fail: 'Not Implemented'! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptPragmaNode: aNode
	self inside: 'pragma' do: [
		super acceptPragmaNode: aNode]! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptReturnNode: aNode
	self inside: 'return' do: [
		super acceptReturnNode: aNode]! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptSequenceNode: aNode
	self inside: 'sequence' do: [
		aNode temporaries ifNotEmpty: [
			self inside: 'args' do: [
				self visitArguments: aNode temporaries]].
		self inside: 'statements' do: [
			aNode statements do: [:each | self visitNode: each]]].! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
acceptVariableNode: aNode
		self text: aNode name asString tagged: 'variable'! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
closeTag
	| name |
	name := tagStack pop.
	
	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self stream nextPutAll:'</';
		nextPutAll: name;
		nextPutAll: '>', Character cr, Character lf.! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
escape: aString
	(1 to: (aString size)) do: [ :index | | c |
		c := aString at: index.
		c := translation at: c ifAbsent: c.
		self stream nextPutAll: c	asString]! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
exportCategories: aList

	self inside: 'categories' do: [
		aList do: [ :item | 
			self exportCategory: item excluding: [:x | false]]]
	
	! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
exportCategory: aString excluding: aBlock

	self inside: 'category'
		with: {'prefix' -> aString}
		do: [self scanCategory: aString excluding: aBlock].! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
exportClass: aClass
	
	self inside: 'class'
		with: {'name' -> aClass name}
		do: [self scanClass: aClass].! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
initialize

	super initialize.
	
	tagStack := Stack new.
	translation := Dictionary new.
	translation
		at: Character cr put: '&#13;';
		at: Character lf put: '&#10;';
		at: Character tab put: '&#9;';
		at: $& put: '&amp;';
		at: $< put: '&lt;';
		at: $> put: '&gt;';
		at: $' put: '&apos;'; 
		at: $" put: '&quot;'.
	! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
inside: aTagName do: aBlock

	self inside: aTagName with: nil do: aBlock.! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
inside: aTagName with: anArgsDict do: aBlock

	(anArgsDict isNil) ifTrue: [
		self openTag: aTagName.]
	ifFalse: [
		self openTag: aTagName withArgs: anArgsDict].
	aBlock value.
	self closeTag.! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
openTag: aName
	
	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	(self stream) 
		nextPut: $< ;
		nextPutAll: aName;
		nextPutAll: '>', Character cr, Character lf.
		
	tagStack push: aName.! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
openTag: aName withArgs: aDictionary

	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self stream nextPut: $< ;
		nextPutAll: aName.
		
	(Dictionary newFrom: aDictionary) keysAndValuesDo: [ :key :value |
		self stream 
			nextPut: $ ;
			nextPutAll: key;
			nextPutAll: '="'.
		self escape: value.
		self stream nextPutAll: '"'].
		
	self stream nextPutAll:  '>', Character cr, Character lf.
		
	tagStack push: aName.! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
scanCategory: aString excluding: aBlock

	((Object allSubclasses
		select: [ :class |
			class category ifNil: [false] ifNotNilDo: [:cat | cat beginsWith: aString]])
		select: [ :class |
			(aBlock value: class) not]) 
		do: [: class |
			self exportClass: class].! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
scanClass: aClass

	aClass methodDict valuesDo: [ :method | 	| node |
		node := RBParser parseMethod: (method getSource asString).
		self visitNode: node.
	]! !

!CMCondenser methodsFor: 'accessing' stamp: 'toma 2/8/2016 11:46'!
stream

	^ stream! !

!CMCondenser methodsFor: 'accessing' stamp: 'toma 2/8/2016 11:46'!
stream: anObject

	stream := anObject! !

!CMCondenser methodsFor: 'accessing' stamp: 'toma 2/8/2016 11:46'!
tagStack

	^ tagStack! !

!CMCondenser methodsFor: 'accessing' stamp: 'toma 2/8/2016 11:46'!
tagStack: anObject

	tagStack := anObject! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
text: aString

	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self escape: aString.
	(self stream)
		nextPut: Character cr;
		nextPut: Character lf.! !

!CMCondenser methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 11:46'!
text: aString tagged: aTagName

	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self stream nextPutAll: '<', aTagName, '>'.
	self escape: aString.
	self stream nextPutAll: '</', aTagName, '>', Character cr, Character lf.! !

RBProgramNodeVisitor subclass: #CMExtractor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMExtractor methodsFor: 'as yet unclassified' stamp: 'toma 2/2/2016 15:48'!
acceptMethodNode: aNode
	
	! !

RBProgramNodeVisitor subclass: #CMLSimpleMiner
	instanceVariableNames: 'model nodestack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

!CMLSimpleMiner methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:58'!
acceptArrayNode: aNode

	^(CMLArray new)
		values: (aNode children collect: [ :each | self visitNode: each]);
		yourself
	! !

!CMLSimpleMiner methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:50'!
mine: anAst
	
	self visitNode: anAst.! !

!CMLSimpleMiner methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:49'!
model

	^ model! !

!CMLSimpleMiner methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:49'!
model: anObject

	model := anObject! !

!CMLSimpleMiner methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:49'!
nodestack

	^ nodestack! !

!CMLSimpleMiner methodsFor: 'accessing' stamp: 'toma 2/8/2016 15:49'!
nodestack: anObject

	nodestack := anObject! !

RBProgramNodeVisitor subclass: #CMLTranslator
	instanceVariableNames: 'collector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-LanguageModel'!

!CMLTranslator methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 16:06'!
acceptArrayNode: anArrayNode
	| array |
	
	array := CMLArray new.
	array
		values: (anArrayNode children collect: [ :each | 
			(self visitNode: each)
				parent: array;
				yourself]).
	self collector addInstance: array.
	^array! !

!CMLTranslator methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 17:10'!
acceptAssignmentNode: anAssignmentNode 
	| assignment |
	assignment := CMLAssignment new.
	assignment
		variable: (anAssignmentNode variable name);
		value: ((self visitNode: (anAssignmentNode value))
						parent: assignment).
	self collector addInstance: assignment.
	^assignment
		! !

!CMLTranslator methodsFor: 'accessing' stamp: 'toma 2/8/2016 17:11'!
collector

	^ collector! !

!CMLTranslator methodsFor: 'accessing' stamp: 'toma 2/8/2016 17:11'!
collector: anObject

	collector := anObject! !

!CMLTranslator methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 16:00'!
translate: anAST

	^self visitNode: anAST! !

!CMLTranslator methodsFor: 'as yet unclassified' stamp: 'toma 2/8/2016 17:13'!
visit: aNode parent: anotherNode
	^(self visitNode: aNode)
			parent: anotherNode;
			yourself.! !

RBProgramNodeVisitor subclass: #CMTextVisitor
	instanceVariableNames: 'literals onSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Text'!

!CMTextVisitor class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 23:18'!
normalize: aString
	"Splits a camel case selector into lowercase atoms without colons"
	
	| tokens lastIndex string |

	string := aString.
	tokens := OrderedCollection new.
	lastIndex := 1.
	string withIndexDo: [ :char :index |
		(char isUppercase or: [char isLetter not]) ifTrue: [
			(index > 1) ifTrue: [
				tokens add: (string copyFrom: lastIndex to: (index - 1)) asLowercase.
			].
			lastIndex := index.
		]
	].
	(lastIndex <= (string size)) ifTrue: [
		tokens add: (string copyFrom: lastIndex to: (string size)) asLowercase.
	].
	^(tokens collect: [:token | token onlyLetters]) select: [:token | token size >= 3]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptLiteralNode: aLiteralNode
	| value |
	super acceptLiteralNode: aLiteralNode.
	value := aLiteralNode token value.
	(value isString)
		ifTrue: [(CMTextVisitor normalize: value) do: [:atom | self literals add: atom]]
	! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/18/2015 18:25'!
acceptMessageNode: aMessageNode 
	
	super acceptMessageNode: aMessageNode.
	(CMTextVisitor normalize: (aMessageNode selector)) do: [ :atom |
		self literals add: atom.
		self onSelector value: (self literals size) value: (aMessageNode selector)].
	self literals add: (aMessageNode selector)! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptMethodNode: aMethodNode
	
	super acceptMethodNode: aMethodNode.
	(CMTextVisitor normalize: (aMethodNode selector)) do: [ :atom |
		self literals add: atom]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptVariableNode: aVariableNode

	super acceptVariableNode: aVariableNode.
	(CMTextVisitor normalize: (aVariableNode name)) do: [ :atom |
		self literals add: atom]
	! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:36'!
initialize 

	super initialize.
	literals := OrderedCollection new.! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:38'!
literals
	^ literals! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:38'!
literals: anObject
	literals := anObject! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/14/2015 16:39'!
onSelector
	^ onSelector! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/14/2015 16:39'!
onSelector: anObject
	onSelector := anObject! !

RBProgramNodeVisitor subclass: #CMTreeExporter
	instanceVariableNames: 'stream tagStack translation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMTreeExporter class methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:16'!
on: aStream

	^(CMTreeExporter new)
		stream: aStream; 
		yourself! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:19'!
acceptArrayNode: anArrayNode
	self inside: 'array' do: [
		super acceptArrayNode: anArrayNode]! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 16:32'!
acceptAssignmentNode: aNode
	self inside: 'assignment' do: [
			self inside: 'to' do: [self visitNode:  aNode variable].
			self inside: 'value' do: [self visitNode: aNode value]]
			! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:26'!
acceptBlockNode: aNode
	self inside: 'block' do: [
		aNode arguments ifNotEmpty: [
		self inside: 'args' do: [
			self visitArguments: aNode arguments]].
		self inside: 'body' do: [
			self visitNode: aNode body.]]! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:26'!
acceptCascadeNode: aNode
	self inside: 'cascade' do: [
		super acceptCascadeNode: aNode]! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:27'!
acceptLiteralArrayNode: aNode
	self inside: 'literals' do: [
		super acceptLiteralArrayNode: aNode.]	! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 17:13'!
acceptLiteralNode: aNode
		self text: (aNode value asString) tagged: 'literal'! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:48'!
acceptMessageNode: aNode
	self inside: 'message' 
	with: {'selector' -> aNode selector asString}
	do: [	
			(aNode isCascaded not or: [aNode isFirstCascaded]) 
				ifTrue: [
					self inside: 'receiver' do:  [
						self visitNode: aNode receiver]].
			aNode arguments ifNotEmpty: [
				self inside: 'args' do: [
					aNode arguments do: [
						:each | self visitNode: each]]]].! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:23'!
acceptMethodNode: aNode
	self inside: 'method' 
	with: {'selector' -> aNode selector asString}
	do: [
		aNode arguments ifNotEmpty: [
			self inside: 'args' do: [
				self visitArguments: aNode arguments]].
		aNode pragmas ifNotEmpty: [
			self inside: 'pragmas' do: [
				aNode pragmas	do: [ :each | self visitNode: each ]]].
		self inside: 'body' do: [
			self visitNode: aNode body]].! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:40'!
acceptPatternBlockNode: aNode
	self fail: 'Not Implemented'! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:40'!
acceptPatternWrapperBlockNode: aNode
	self fail: 'Not Implemented'! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:41'!
acceptPragmaNode: aNode
	self inside: 'pragma' do: [
		super acceptPragmaNode: aNode]! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:41'!
acceptReturnNode: aNode
	self inside: 'return' do: [
		super acceptReturnNode: aNode]! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:26'!
acceptSequenceNode: aNode
	self inside: 'sequence' do: [
		aNode temporaries ifNotEmpty: [
			self inside: 'args' do: [
				self visitArguments: aNode temporaries]].
		self inside: 'statements' do: [
			aNode statements do: [:each | self visitNode: each]]].! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 16:36'!
acceptVariableNode: aNode
		self text: aNode name asString tagged: 'variable'! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:32'!
closeTag
	| name |
	name := tagStack pop.
	
	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self stream nextPutAll:'</';
		nextPutAll: name;
		nextPutAll: '>', Character cr, Character lf.! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 17:04'!
escape: aString
	(1 to: (aString size)) do: [ :index | | c |
		c := aString at: index.
		c := translation at: c ifAbsent: c.
		self stream nextPutAll: c	asString]! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 17:27'!
exportCategories: aList

	self inside: 'categories' do: [
		aList do: [ :item | 
			self exportCategory: item excluding: [:x | false]]]
	
	! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:15'!
exportCategory: aString excluding: aBlock

	self inside: 'category'
		with: {'prefix' -> aString}
		do: [self scanCategory: aString excluding: aBlock].! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:12'!
exportClass: aClass
	
	self inside: 'class'
		with: {'name' -> aClass name}
		do: [self scanClass: aClass].! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 17:01'!
initialize

	super initialize.
	
	tagStack := Stack new.
	translation := Dictionary new.
	translation
		at: Character cr put: '&#13;';
		at: Character lf put: '&#10;';
		at: Character tab put: '&#9;';
		at: $& put: '&amp;';
		at: $< put: '&lt;';
		at: $> put: '&gt;';
		at: $' put: '&apos;'; 
		at: $" put: '&quot;'.
	! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:19'!
inside: aTagName do: aBlock

	self inside: aTagName with: nil do: aBlock.! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 16:18'!
inside: aTagName with: anArgsDict do: aBlock

	(anArgsDict isNil) ifTrue: [
		self openTag: aTagName.]
	ifFalse: [
		self openTag: aTagName withArgs: anArgsDict].
	aBlock value.
	self closeTag.! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:32'!
openTag: aName
	
	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	(self stream) 
		nextPut: $< ;
		nextPutAll: aName;
		nextPutAll: '>', Character cr, Character lf.
		
	tagStack push: aName.! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 17:10'!
openTag: aName withArgs: aDictionary

	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self stream nextPut: $< ;
		nextPutAll: aName.
		
	(Dictionary newFrom: aDictionary) keysAndValuesDo: [ :key :value |
		self stream 
			nextPut: $ ;
			nextPutAll: key;
			nextPutAll: '="'.
		self escape: value.
		self stream nextPutAll: '"'].
		
	self stream nextPutAll:  '>', Character cr, Character lf.
		
	tagStack push: aName.! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:13'!
scanCategory: aString excluding: aBlock

	((Object allSubclasses
		select: [ :class |
			class category ifNil: [false] ifNotNilDo: [:cat | cat beginsWith: aString]])
		select: [ :class |
			(aBlock value: class) not]) 
		do: [: class |
			self exportClass: class].! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 1/29/2016 18:11'!
scanClass: aClass

	aClass methodDict valuesDo: [ :method | 	| node |
		node := RBParser parseMethod: (method getSource asString).
		self visitNode: node.
	]! !

!CMTreeExporter methodsFor: 'accessing' stamp: 'toma 1/29/2016 16:08'!
stream

	^ stream! !

!CMTreeExporter methodsFor: 'accessing' stamp: 'toma 1/29/2016 16:08'!
stream: anObject

	stream := anObject! !

!CMTreeExporter methodsFor: 'accessing' stamp: 'toma 1/29/2016 16:08'!
tagStack

	^ tagStack! !

!CMTreeExporter methodsFor: 'accessing' stamp: 'toma 1/29/2016 16:08'!
tagStack: anObject

	tagStack := anObject! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 17:03'!
text: aString

	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self escape: aString.
	(self stream)
		nextPut: Character cr;
		nextPut: Character lf.! !

!CMTreeExporter methodsFor: 'as yet unclassified' stamp: 'toma 2/1/2016 16:59'!
text: aString tagged: aTagName

	(1 to: (tagStack size)) do: [ :i | self stream nextPutAll: '  '].	
	self stream nextPutAll: '<', aTagName, '>'.
	self escape: aString.
	self stream nextPutAll: '</', aTagName, '>', Character cr, Character lf.! !
